import bpy

from . import OperatorWithOptions, get_selected
from .. import scene, util
from ..vendor import bpx


class CreateAttachConstraint(OperatorWithOptions):
    bl_idname = "ragdoll.create_attach_constraint"
    bl_label = "Create Attach Constraint"
    bl_options = {"REGISTER", "UNDO"}
    bl_description = "Create a new solver"

    @classmethod
    def poll(cls, context):
        solver = bpx.ls(type="rdSolver")

        if not solver:
            cls.poll_message_set("No solver in the scene")
            return False

        return True

    def execute(self, context):
        solver = scene.find_or_create_current_solver()

        markers = get_selected("rdMarker")

        if len(markers) != 2:
            self.report({"ERROR"}, "Select 2 markers to attach")
            return {"CANCELLED"}

        parent, child = markers

        xparent = bpx.alias(parent)
        xchild = bpx.alias(child)

        xcon = scene.create("rdPinConstraint", name="rAttachConstraint")
        xcon["parentMarker"] = xparent.handle()
        xcon["childMarker"] = xchild.handle()

        # More suitable default values
        xcon["linearStiffness"] = 0.1
        xcon["angularStiffness"] = 0.1

        # TODO: Make dependent on option
        transform = xchild["sourceTransform"].read()
        xcon.handle().matrix_world = transform.matrix()

        # Make it act like a child
        xsource = xparent["sourceTransform"].read()
        childof = bpx.create_constraint(xcon, "CHILD_OF")
        childof.target = xsource.handle()

        if isinstance(xsource, bpx.BpxBone):
            childof.subtarget = xsource.pose_bone().name

        # Append to solver
        solver["members"].append({"object": xcon.handle()})
        bpx.link(xcon, util.find_assembly())

        with bpx.object_mode():
            bpx.select(xcon)

        util.touch_initial_state(context)

        return {"FINISHED"}


def install():
    bpy.utils.register_class(CreateAttachConstraint)


def uninstall():
    bpy.utils.unregister_class(CreateAttachConstraint)
