import bpy
from . import tag_redraw
from ..vendor import bpx
from .. import util

_physics_types = (
    "rdSolver",
    "rdMarker",
    "rdGroup",
    "rdPinConstraint",
    "rdFixedConstraint",
    "rdDistanceConstraint",
)


def delete_physics(xobjs):
    bpx.delete(*xobjs)
    util.touch_initial_state()


def delete_physics_all():
    bpx.delete(*bpx.ls(type=_physics_types))
    util.touch_initial_state()


class DeletePhysicsByEntity(bpy.types.Operator):
    bl_idname = "ragdoll.delete_physics_by_entity"
    bl_label = "Delete Physics By Id"
    bl_options = {"REGISTER", "UNDO"}
    bl_description = "Delete one Ragdoll entity"

    entity: bpy.props.IntProperty(options={"SKIP_SAVE"})

    @classmethod
    def poll(cls, context):
        if context.mode == "OBJECT":
            return True
        cls.poll_message_set("Need to be in Object Mode")
        return False

    def execute(self, context):
        xobj = bpx.alias(self.entity, None)
        if xobj is None:
            return {"CANCELLED"}

        delete_physics([xobj])
        tag_redraw(context.screen)
        return {"FINISHED"}


class DeletePhysicsBySelection(bpy.types.Operator):
    bl_idname = "ragdoll.delete_physics_by_selection"
    bl_label = "Delete Physics from Selection"
    bl_options = {"REGISTER", "UNDO"}
    bl_description = "Delete selected Ragdoll physics in scene"

    @classmethod
    def poll(cls, context):
        if context.mode == "OBJECT":
            return True
        cls.poll_message_set("Need to be in Object Mode")
        return False

    def execute(self, context):
        delete_physics(bpx.selection(type=_physics_types))
        tag_redraw(context.screen)
        return {"FINISHED"}


class DeletePhysicsAll(bpy.types.Operator):
    bl_idname = "ragdoll.delete_physics_all"
    bl_label = "Delete All Physics"
    bl_options = {"REGISTER", "UNDO"}
    bl_description = "Delete all Ragdoll physics in scene"

    @classmethod
    def poll(cls, context):
        if context.mode == "OBJECT":
            return True
        cls.poll_message_set("Need to be in Object Mode")
        return False

    def execute(self, context):
        delete_physics_all()
        tag_redraw(context.screen)
        return {"FINISHED"}


def install():
    bpy.utils.register_class(DeletePhysicsByEntity)
    bpy.utils.register_class(DeletePhysicsBySelection)
    bpy.utils.register_class(DeletePhysicsAll)


def uninstall():
    bpy.utils.unregister_class(DeletePhysicsByEntity)
    bpy.utils.unregister_class(DeletePhysicsBySelection)
    bpy.utils.unregister_class(DeletePhysicsAll)
