import os
import sys
import bpy.utils.previews  # noqa

_icons = {}


def install():
    dirname = os.path.dirname(os.path.dirname(__file__))  # ragdoll
    dirname = os.path.join(dirname, "resources", "icons")

    icons = [
        ("logo", "logo.png"),
        ("logo2", "logo2.png"),
        ("record", "record.png"),
        ("ctrl", "ctrl-mac.png") if sys.platform == "darwin" else
        ("ctrl", "ctrl.png"),
    ]
    collection = bpy.utils.previews.new()  # Note: a subclass of dict

    for name, fname in icons:
        filepath = os.path.join(dirname, fname)

        collection.load(name, filepath, "IMAGE")

        # Note: Accessing the 0-th index somehow improves quality
        collection[name].icon_size[0]

    global _icons
    _icons = collection


def uninstall():
    try:
        bpy.utils.previews.remove(_icons)
    except AttributeError:
        pass

    _icons.clear()


def ls() -> list[str]:
    """Returns a list of loaded icon names"""
    loaded_icons: dict = _icons
    return sorted(loaded_icons.keys())


def name_to_icon_id(name: str) -> int:
    """Get icon_id by `name`"""
    return getattr(_icons.get(name), "icon_id", 0)


def name_to_icon_path(name: str) -> str:
    dirname = os.path.dirname(os.path.dirname(__file__))  # ragdoll
    dirname = os.path.join(dirname, "resources", "icons")
    return os.path.join(dirname, name)
